# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - MoinRemoteSupport library

    @copyright: 2011, 2012, 2013 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from ContentTypeSupport import getContentTypeAndEncoding
from MoinMoin.action import cache
from MoinMoin import caching
import urllib2, time

def getCachedResource(request, url, arena, scope, max_cache_age):

    """
    Using the given 'request', return the resource data for the given 'url',
    accessing a cache entry with the given 'arena' and 'scope' where the data
    has already been downloaded. The 'max_cache_age' indicates the length in
    seconds that a cache entry remains valid.

    If the resource cannot be downloaded and cached, None is returned.
    Otherwise, the form of the data is as follows:

    url <newline>
    [ content-type-header ] <newline>
    [ other-header <newline> ]*
    <newline>
    content-body
    """

    # See if the URL is cached.

    cache_key = cache.key(request, content=url)
    cache_entry = caching.CacheEntry(request, arena, cache_key, scope=scope)

    # If no entry exists, or if the entry is older than the specified age,
    # create one with the response from the URL.

    now = time.time()
    mtime = cache_entry.mtime()

    # NOTE: The URL could be checked and the 'If-Modified-Since' header
    # NOTE: (see MoinMoin.action.pollsistersites) could be checked.

    if not cache_entry.exists() or now - mtime >= max_cache_age:

        # Access the remote data source.

        cache_entry.open(mode="w")

        try:
            f = urllib2.urlopen(url)
            try:
                cache_entry.write(url + "\n")
                cache_entry.write((f.headers.get("content-type") or "") + "\n")
                for key, value in f.headers.items():
                    if key.lower() != "content-type":
                        cache_entry.write("%s: %s\n" % (key, value))
                cache_entry.write("\n")
                cache_entry.write(f.read())
            finally:
                cache_entry.close()
                f.close()

        # In case of an exception, return None.

        except IOError:
            if cache_entry.exists():
                cache_entry.remove()
            return None

    # Open the cache entry and read it.

    cache_entry.open()
    try:
        return cache_entry.read()
    finally:
        cache_entry.close()

def getCachedResourceMetadata(f):

    "Return a metadata dictionary for the given resource file-like object 'f'."

    url = f.readline()
    content_type, encoding = getContentTypeAndEncoding(f.readline())

    metadata = {}
    line = f.readline()

    while line.strip():
        key, value = [v.strip() for v in line.split(":", 1)]
        metadata[key] = value
        line = f.readline()

    return url, content_type, encoding, metadata

# vim: tabstop=4 expandtab shiftwidth=4
